/* Autogenerated by waf */
#define SAMBA_COPYRIGHT_STRING "Copyright Andrew Tridgell and the Samba Team 1992-2026"
#define SAMBA_VERSION_MAJOR 4
#define SAMBA_VERSION_MINOR 24
#define SAMBA_VERSION_RELEASE 4
#define SAMBA_VERSION_OFFICIAL_STRING "4.24.4"

#ifdef SAMBA_VERSION_VENDOR_FUNCTION
#  define SAMBA_VERSION_STRING SAMBA_VERSION_VENDOR_FUNCTION
#else /* SAMBA_VERSION_VENDOR_FUNCTION */
#  define SAMBA_VERSION_STRING "4.24.4"
#endif
/* Version for mkrelease.sh: 
SAMBA_VERSION_STRING=4.24.4
 */
