/* header auto-generated by pidl */

#ifndef _PIDL_HEADER_smb3posix
#define _PIDL_HEADER_smb3posix

#include <stdint.h>


#include <gen_ndr/security.h>
#ifndef _HEADER_smb3posix
#define _HEADER_smb3posix

#include <util/time.h>

#define UNIX_TYPE_FILE	( 0 )
#define UNIX_TYPE_DIR	( 1 )
#define UNIX_TYPE_SYMLINK	( 2 )
#define UNIX_TYPE_CHARDEV	( 3 )
#define UNIX_TYPE_BLKDEV	( 4 )
#define UNIX_TYPE_FIFO	( 5 )
#define UNIX_TYPE_SOCKET	( 6 )
#define UNIX_TYPE_UNKNOWN	( 0xFFFFFFFF )
#define UNIX_X_OTH	( 0000001 )
#define UNIX_W_OTH	( 0000002 )
#define UNIX_R_OTH	( 0000004 )
#define UNIX_X_GRP	( 0000010 )
#define UNIX_W_GRP	( 0000020 )
#define UNIX_R_GRP	( 0000040 )
#define UNIX_X_USR	( 0000100 )
#define UNIX_W_USR	( 0000200 )
#define UNIX_R_USR	( 0000400 )
#define UNIX_STICKY	( 0001000 )
#define UNIX_SET_GID	( 0002000 )
#define UNIX_SET_UID	( 0004000 )
#define UNIX_OTH_MASK	( 0000007 )
#define UNIX_GRP_MASK	( 0000070 )
#define UNIX_USR_MASK	( 0000700 )
#define UNIX_PERM_MASK	( 0000777 )
#define UNIX_EXTRA_MASK	( 0007000 )
#define UNIX_FILETYPE_MASK	( 0070000 )
#define UNIX_FILETYPE_SHIFT	( 12 )
#define UNIX_ALL_MASK	( 0077777 )
struct smb3_posix_cc_info {
	uint32_t nlinks;
	uint32_t reparse_tag;
	uint32_t posix_mode;
	struct dom_sid owner;
	struct dom_sid group;
}/* [flag(LIBNDR_FLAG_NOALIGN),public] */;

struct smb3_file_posix_information {
	NTTIME creation_time;
	NTTIME last_access_time;
	NTTIME last_write_time;
	NTTIME change_time;
	uint64_t end_of_file;
	uint64_t allocation_size;
	uint32_t file_attributes;
	uint64_t inode;
	uint32_t device;
	uint32_t reserved;
	struct smb3_posix_cc_info cc;
}/* [flag(LIBNDR_FLAG_NOALIGN),public] */;

#endif /* _HEADER_smb3posix */
#endif /* _PIDL_HEADER_smb3posix */
